from typing import List, Optional, Any
from freesif import open_sif    
import shutil
import numpy as np
import sys
import os
import uuid
import random
import string

def id_generator(size:int=6, chars:str=string.ascii_uppercase + string.digits) -> str:
    """Generates a random string for use as a unique identifier (id).

    Args:
        size (int, optional): Length of random string (id) to be generated. Defaults to 6.
        chars (str, optional): String from which to randomly extract characters for the id. Defaults to string.ascii_uppercase+string.digits.

    Returns:
        str: id
        
    TODO: Move to an external library
    """
    return ''.join(random.choice(chars) for _ in range(size))


def get_wadam_strings(folder:str, filename:str, wdfactors:List[float]=[1]*6, 
    pmin:List[float]=[10]*6, pmax:List[float]=[30]*6, ppeak:List[float]=[18.0]*6, 
    scaling:List[str] = ["cos2"]*6, return_arrays:bool=False, 
    return_hz_dirs:bool=False)->List[List[Any]]:
    """Reads a WADAM SIF file and returns [raoStrings, wavedriftStrings]
    which are lists of length 6, where the 6 entries relate to the degrees of 
    freedom of body motion (surge, sway, heave, roll, pitch, yaw) respectively. 
    
    Each entry is a string which is in SIMO (sys.dat) format, where e.g.
        raoStrings[0] is the surge RAO as a single string
        wavedriftStrings[3] is the mean wavedrift coefficients in roll.
        
    In addition to the strings, the user can get the coefficients as value arrays,
    and can request the frequencies (Hz) and directions (degrees)::
    
        raoStr, wdStr = get_wadam_strings(folder,filename)
        freqs, dirs, raoStr, wdStr = get_wadam_strings(folder,filename,return_hz_dirs=True)
        freqs, dirs, raoStr, wdStr, raos, wds = get_wadam_strings(folder,filename,return_hz_dirs=False)      

    Optionally the user can scale the wave drift coefficients using a scaling function.
    The scaling function is specified by the "scaling" argument and can take the values:
    
        "constant": constant scaling
        "linear":   linear scaling. At wave periods <= pmin and >= pmax the scaling factor takes the value of 1.0
                    i.e. no scaling. At wave period ppeak, where pmin < ppeak < pmax, the scaling factor takes a
                    value specified by the corresponding entry in the wdfactors list for the response.
        "cos2" :    Similar to linear except the scaling function between pmin/pmax and ppeak is cosine-squared
        "gauss":    Similar to cos2 except the scaling function is Guassian and the scaling values at pmin/pmin
                    10% the value specified by the wdfactor array

    Args:
        folder (str): folder of the SIF file
        filename (str): filename of the SIF file
        wdfactors (List[float], optional): Maximum value of scaling function to apply at ppeak. Defaults to [1]*6.
        pmin (List[float], optional): Minimum period for scaling. Defaults to [10]*6.
        pmax (List[float], optional): Maximum period for scaling. Defaults to [30]*6.
        ppeak (List[float], optional): Peak period for scaling. Defaults to [18.0]*6.
        scaling (List[str], optional): Type of scaling. Defaults to ["cos2"]*6.
        return_arrays (bool, optional): Set true to return values of raos and wavedrift coefficients. Defaults to False.
        return_hz_dirs (bool, optional): Set True to return wave frequencies (Hz) and directions. Defaults to False.

    Returns:
        List[List[Any]]: See docstring
    """
    # Read the mean wave drift coefficients and raos on the SIF file
    id = id_generator()
    filename_tmp = "r" + id + filename[:-4].replace(".","_") + '.SIF'
    shutil.copyfile(f"{folder}/{filename}", filename_tmp)
    # Open the sif and read the data
    sif = open_sif(filename_tmp)
    periods = sif.get_periods()
    directions = sif.get_directions()
    raos = sif.get_motion_raos()
    pressure = sif.get_meandrift()
    momentum = sif.get_horiz_meandrift()
    raos = raos[:, :, ::-1] # flip the last field
    momentum = momentum[:,:,::-1]
    pressure = pressure[:,:,::-1]
    periods = periods[::-1] # for consistency (superfluous for function)
    wavedrift = pressure
    wavedrift[0] = momentum[0]
    wavedrift[1] = momentum[1]
    wavedrift[5] = momentum[2]

    # Establish some blending functions to scale the wavedrifts
    alpha = np.zeros((6,len(periods))) # storage for blending function
    for i in range(0, 6):
        print(i, scaling[i])
        ix_lower = (periods >= pmin[i]) & (periods <= ppeak[i]) 
        ix_upper = (periods <= pmax[i]) & (periods >= ppeak[i])
        coord_lower = 1-(ppeak[i]-periods[ix_lower])/(ppeak[i] - pmin[i])
        coord_upper = 1-(periods[ix_upper]-ppeak[i])/(pmax[i] - ppeak[i])
        if scaling[i] == "constant":
            alpha[i,:] = 1
        if scaling[i] == "linear":
            alpha[i,ix_lower] = coord_lower
            alpha[i,ix_upper] = coord_upper
        if scaling[i] == "cos2":
            alpha[i,ix_lower] = (0.5*(1 - np.cos(coord_lower*np.pi)))**2
            alpha[i,ix_upper] = (0.5*(1 - np.cos(coord_upper*np.pi)))**2
        if scaling[i] == "gauss":
            fwtm_lower = 2*(ppeak[i]-pmin[i]) # full width at tenth of the maximum
            fwtm_upper = 2*(pmax[i]-ppeak[i]) # full width at tenth of the maximum
            c_lower = fwtm_lower/4.29193
            c_upper = fwtm_upper/4.29193
            alpha_lower = np.exp(-(periods-ppeak[i])**2/2/c_lower**2)
            alpha_upper = np.exp(-(periods-ppeak[i])**2/2/c_upper**2)
            alpha[i] = np.where(periods < ppeak[i], alpha_lower, alpha_upper)
        if scaling[i] == "exwave":
            # TODO: requires hs snd other arguments, disallow for now
            hs = 12.0
            D = 96
            U = 0.73 # would need to enter
            G = 10 # s
            Cp = 0.25
            w = 2*np.pi/periods
            k = w**2/9.80665
            p = np.exp(-1.25*(k*D)**2)
            Dsum = D
            B = k*p*Dsum
            fd2 = B*(G*U +  hs)
            alpha[i] = (1+Cp*U)   

    # if the factor is 1.0 want no change. If factor 1.1 we want some change
    wavedrift_original = wavedrift*1.0 # make a copy for checking (then remove!)
    wavedrift_peak = wavedrift*1.0 # make storage for peak scaling
    for idof in range(len(raos)):
        for idir in range(len(directions)):
            if scaling[idof] == "exwave":
                #wavedrift[idof,idir] = wavedrift_original[idof,idir] + 1.0*B*(G*U +  hs)*1000
                #wavedrift[idof,idir] = wavedrift_original[idof,idir] + 1.0*B*(26.4)*1000
                wavedrift[idof,idir] = wavedrift_original[idof,idir] + B*1000*wdfactors[idof]
            else:
                wavedrift_peak[idof,idir] = wavedrift_original[idof,idir]*wdfactors[idof]
                wavedrift[idof,idir] = alpha[idof]*wavedrift_peak[idof,idir] + (1-alpha[idof])*wavedrift_original[idof,idir]

    # Establish storage
    raoStrings = [[],[],[],[],[],[]]
    wavedriftStrings = [[],[],[],[],[],[]]
    for idof in range(len(raos)):
        for idir in range(len(directions)):
            for ifreq in range(len(periods)):
                h = raos[idof,idir,ifreq]
                a, p = np.abs(h), np.angle(h)*180/np.pi
                v = wavedrift[idof,idir,ifreq] / 1000 # convert from N => kN 
                raoStrings[idof].append(f"{idir+1:5d} {ifreq+1:5d} {a:20.9e} {p:20.9e}") 
                wavedriftStrings[idof].append(f"{idir+1:5d} {ifreq+1:5d} {v:20.9e}") 

    sif.close()
    os.remove(filename_tmp)
    dataset = [1/periods, directions] if return_hz_dirs else []
    dataset += [raoStrings, wavedriftStrings]
    dataset += [raos, wavedrift] if return_arrays else []
    return dataset



def read_SIF_rao(folder:str, filename:str, wadir:float, 
                 mapping_index:List[int]=[0,1,2,3,4,5],
                 mapping_sign:List[int]=[1,1,1,1,1,1]):
    """Returns the RAOs on the SIF file for a given wave direction.

    Args:
        folder (str): folder containing the SIF file
        filename (str): filename of the SIF file
        wadir (float): wave direction of interest
        mapping_index (List[int], optional): Maps the degrees of freedom to another order. Defaults to [1,0,2,4,3,5].

    Returns:
        [type]: [description]
    """
    
    # Need to replace prefix dots (.) due to REGEX method in freesif
    id = id_generator()
    filename_tmp =  "r" + id + filename[:-4].replace(".","_") + '.SIF'
    shutil.copyfile(f"{folder}/{filename}", filename_tmp)
    sif = open_sif(filename_tmp)
    periods = sif.get_periods()
    directions = sif.get_directions()
    raos = sif.get_motion_raos()
    sif.close()
    os.remove(filename_tmp)
    ix = np.nonzero(wadir==directions)
    factor = [1,1,1,180./np.pi,180./np.pi,180./np.pi]
    # Correcting for the chosen coordinate system :/
    return 1./periods[::-1], [mapping_sign[dof]*factor[dof]*np.squeeze(raos[dof, ix, ::-1]) for dof in mapping_index]


import numpy as np
from abc import ABC, abstractmethod, abstractproperty

def wave_params(Ap=5, Tp=10, h=1000):
    """
        DNV C205 parameters for assessing regular waves
    """
    g = 9.80665 # m/s/s
    Lp = Tp**2*g/2/np.pi
    H = 2*Ap
    h =  min(h, 0.8*Lp) # m TODO: (remove - temporary hack for CFD comparisons)
    w = 2*np.pi/Tp
    k = w**2/g # be lazy (apply deep water relation)                                              
    s = np.tanh(k*h)
    stokes_2nd = 1 + k*Ap*(3-s**2)/4/s**3
    wave_steepness_param = S = H/Lp
    shallow_water_param = mu = h/Lp
    ursells_nr = ur = S/mu**3 # H*Lp**2/h**3
    return stokes_2nd, S, mu, ur

class RegularWave(ABC):
    """ Abstract class for regular waves """
    @abstractproperty
    def height(self): pass
    @abstractproperty
    def period(self): pass
    @abstractproperty
    def phase(self): pass
    @abstractproperty
    def wavelength(self): pass
    @abstractproperty
    def wavenumber(self): pass
    @abstractproperty
    def crest_level(self): pass
    @abstractproperty
    def trough_level(self): pass
    @abstractproperty
    def group_velocity(self): pass
    @abstractproperty
    def phase_velocity(self): pass


class StokesFirst(RegularWave):
    """
        First-order Stokes wave implementation
    """
    def __init__(self, height, period, phase=0, depth=1000, g=9.80665):
        self.__gravity = g
        self.__depth = depth
        self.__height = height
        self.__period = period
        self.__phase = phase
        self.__wavenumber = self.__compute_wavenumber()
        k, d = self.__wavenumber, depth
        self.__wavelength = 2*np.pi/k
        self.__crest_level = height/2
        self.__trough_level = height/2
        self.__phase_velocity = (g/k*np.tanh(k*d))**0.5
        self.__group_velocity = self.phase_velocity*(1+2*k*d/np.sinh(2*k*d))

    @property
    def height(self): return self.__height
    @property
    def period(self): return self.__period
    @property
    def phase(self): return self.__phase
    @property
    def wavelength(self): return self.__wavelength
    @property
    def wavenumber(self): return self.__wavenumber
    @property
    def crest_level(self): return self.__crest_level
    @property
    def trough_level(self): return self.__trough_level
    @property
    def group_velocity(self): return self.__group_velocity
    @property
    def phase_velocity(self): return self.__phase_velocity

    def surface_elevation(self, x, t):
        theta = self.__wavenumber*(x-self.__phase_velocity*t) + self.phase
        return self.__height/2*np.cos(theta)

    def horizontal_velocity():
        pass

    #@classmethod
    def __compute_wavenumber(self, g = 9.80665):
        """ Returns the wave number satisfying the first order linear dispersion relation 
            w**2 = gktanh(kh)
        """
        from scipy.optimize import root_scalar
        # To relax requirement on input allow for both iterable and non-iterable inputs
        wi = 2*np.pi / self.__period
        h = self.__depth
        sol = root_scalar(lambda k_, h, w: w**2 - g*k_*np.tanh(k_*h), 
                                args=(h, wi), method='brentq', bracket=[0, 10])
        if not sol.converged: print(f"Period {2*np.pi/wi} is not converged.")
        return sol.root


class StokesSecond(RegularWave):
    """
        Second-order Stokes wave implementation
    """
    def __init__(self, height, period, phase=0, depth=1000, g=9.80665):
        self.__gravity = g
        self.__depth = depth
        self.__height = height
        self.__period = period
        self.__phase = phase
        self.__wavenumber = self.__compute_wavenumber()
        k, d, a = self.__wavenumber, depth, height/2
        s = np.tanh(k*d)
        self.__wavelength = 2*np.pi/k
        self.__crest_level = a * (1 + k*a*(3-s**2)/(4*s**3))
        self.__trough_level = -a * (1 - k*a*(3-s**2)/(4*s**3))
        self.__phase_velocity = (g/k*np.tanh(k*d))**0.5
        self.__group_velocity = self.phase_velocity*(1+2*k*d/np.sinh(2*k*d))

    @property
    def height(self): return self.__height
    @property
    def period(self): return self.__period
    @property
    def phase(self): return self.__phase
    @property
    def wavelength(self): return self.__wavelength
    @property
    def wavenumber(self): return self.__wavenumber
    @property
    def crest_level(self): return self.__crest_level
    @property
    def trough_level(self): return self.__trough_level
    @property
    def group_velocity(self): return self.__group_velocity
    @property
    def phase_velocity(self): return self.__phase_velocity

    def surface_elevation(self, x, t):
        theta = self.__wavenumber*(x-self.__phase_velocity*t) + self.phase
        kd, H, L = self.__wavenumber*self.__depth, self.__height, self.__wavelength
        linear = H/2*np.cos(theta)
        second = np.pi*H**2/8/L*np.cosh(kd)/np.sinh(kd)**3*(2+np.cosh(2*kd))*np.cos(2*theta)
        return linear + second

    #@classmethod
    def __compute_wavenumber(self):
        """ Returns the wave number satisfying the first order linear dispersion relation 
            w**2 = gktanh(kh)
        """
        from scipy.optimize import root_scalar
        # To relax requirement on input allow for both iterable and non-iterable inputs
        wi = 2*np.pi / self.__period
        h = self.__depth
        g = self.__gravity
        sol = root_scalar(lambda k_, h, w: w**2 - g*k_*np.tanh(k_*h), 
                                args=(h, wi), method='brentq', bracket=[0, 10])
        if not sol.converged: print(f"Period {2*np.pi/wi} is not converged.")
        return sol.root



if __name__ == '__main__':
    import matplotlib.pyplot as plt
    first_order = StokesFirst(height=5, period=10, depth=300)
    second_order = StokesSecond(height=5, period=10, depth=300)
    print(first_order.wavelength)
    print(first_order.wavenumber, 2*np.pi/(first_order.wavenumber*9.80665)**.5, first_order.wavelength/first_order.phase_velocity)
    print(first_order.phase_velocity)
    print(first_order.phase_velocity*first_order.period)

    print(first_order.crest_level)
    print(second_order.crest_level, second_order.trough_level)
    print(second_order.crest_level - second_order.trough_level)

    x = np.linspace(0, 2*second_order.wavelength, 1000)
    plt.plot(x, second_order.surface_elevation(x,t=0), label="second")
    plt.plot(x, first_order.surface_elevation(x,t=0), label="first")
    plt.axhline(second_order.crest_level)
    plt.legend(loc=0)
    plt.show()



r"""A python module for administration tools
    """
  
import subprocess 
import sys
import os

def multipleReplace(text, wordDict):
    """
    take a text and replace words that match the key in a dictionary
    with the associated value, return the changed text
    """
    for key in wordDict:
        text = text.replace(key, wordDict[key])
    return text

def runprocess(cmd,returnstdout=True):
    print("Running %s" % cmd)
    if (returnstdout):
        process = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
        output = process.communicate()[0]
        output = output.split('\n')
        return output
    else:
        process = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
        while 1:
            output = process.stdout.readline()[:-1]
            print(output)
            if output == '': return None



def docx_replace(old_file, wordDict, new_file):
    import zipfile
    import shutil
    # This allows the old and new file to be identical
    temporary_file = "tmp.docx"
    shutil.copyfile(old_file,temporary_file)    
    zin  = zipfile.ZipFile (temporary_file, 'r')
    zout = zipfile.ZipFile (new_file,       'w')

    for item in zin.infolist():
        buffer = zin.read(item.filename)

        if (item.filename == 'word/document.xml'):
            res = buffer.decode("utf-8")
            for keyword in wordDict:
                res = res.replace(keyword,wordDict[keyword])
            buffer = res.encode("utf-8")

        if (item.filename == 'word/header1.xml'):
            res = buffer.decode("utf-8")
            for keyword in wordDict:
                res = res.replace(keyword,wordDict[keyword])
            buffer = res.encode("utf-8")

        zout.writestr(item, buffer)

    zout.close()
    zin.close()



def sed(infile, wordDict, outfile, delimiter=';'):
    """
    Replace text in infile and replace words that match the 
    key in a dictionary with the associated value.
    Output new text to outfile. 
    Operation similar to 'seds' basic string operation
    """
    text = delimiter.join(open(infile,'r').readlines())
    for key in wordDict:
        text = text.replace(key, str(wordDict[key]))
    open(outfile,'w').writelines(text.split(delimiter))

def ssh_connect(host,password):
    ssh = paramiko.SSHClient()
    ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy()) 
    ssh.connect('%s.marintek.sintef.no' % host,
                username='tken', password=password)
    return ssh


def find_occurrences(string, lines, specificOccurrenceWanted = False):
    try:
        occurrences = \
            [loc for loc, line in enumerate(lines)  \
                 if string in line]
        if specificOccurrenceWanted:
            return occurrences[specificOccurrenceWanted-1]
        else: return occurrences

    except Exception:
        print("Can't find %d occurrence %s in file lines" % \
            (specificOccurrenceWanted, string))
        sys.exit(-1)


def timer(func):
    """
        This function shows the execution time of 
        the function object passed
    """
    from time import time
    def wrap_func(*args, **kwargs):
        t1 = time()
        result = func(*args, **kwargs)
        t2 = time()
        print(f'Function {func.__name__!r} executed in {(t2-t1):.4f}s')
        return result
    return wrap_func


def getpassword():
    """ Script requires a password """
    text = "Enter password to run script"
    try:
        return getpass.unix_getpass(prompt=text)
    except:
        try:
            return getpass.win_getpass(prompt=text)
        except:
            sys.stderr.write('Error in getting password')
            sys.stderr.flush()
            sys.exit(-1)


def formatwrite(f, L,width=11,usef=True):  
    #print L
    f.write('%-1s' % L[0])
    for listitem in L[1:]:
        if isinstance(listitem,str):   format ="%"+ "-%ds" % width
        if isinstance(listitem,int):   format ="%"+ "-%dd" % width
        if isinstance(listitem,float): 
            if (usef): format ="%"+ "-%d.3f" % width
            else:      format ="%"+ "-%d.3e" % width
        f.write(format % listitem)
    f.write('\n')

   
def ntasks(process):
    """ -1 is required as to remove the empty line"""
    output = runprocess('tasklist /v | findstr %s' % process)
    return len(output)-1
    

def winhold(process,limit):
    """ -1 is required as to remove the empty line"""
    from time import sleep
    print(ntasks(process))
    while ntasks(process) > limit: sleep(6)


def ntasks_linux():
    """ -1 is required as to remove the empty line"""
    output = runprocess('qstat -r -u tike')
    return max(0,len(output)-1 -5)


def linuxhold(limit):
    """
    """
    from time import sleep
    import os
    while ntasks_linux() > limit: sleep(6)


def mkdir(folder):
    try: os.mkdir(folder)
    except: pass

    
    



r"""A python module with various tools for time series

    Functions:
    
    timetools.resample
    timetools.window
    timetools.make_plotMT
    timetools.allstats
    

    """


import numpy as np
import pylab as plt
import math
from scipy import stats as scipystats



###########################################################
###########################################################



def resample(x,told,dtnew):
    r"""timetools.resample
        -----------------
    
        Resamples the time series x over a new time vector
        
        vak 19.05.2011

        -----------------

        Input:   x     - time series 
                 told  - old time vector
                 dtnew - new time step                 

        Output:  xnew  - resampled time series
                 tnew  - corresponding time vector

        -----------------"""

    # Time vector characteristics
    #dtold = told[1]-told[0]
    tnew = np.arange(told[0],told[-1]+dtnew/2.,dtnew)
    #print len(tnew), len(told), len(x)
    
    # Linearly interpolate x over tnew
    xnew = np.interp(tnew,told,x)

    return xnew, tnew



###########################################################
###########################################################

    
def plot_ts(title,x0,y0,label,unit,tofile):
    plt.figure(figsize=(12,8))
    plt.axes([0.15,0.1,0.65,0.75])
    for i in range(0,3):

        plt.subplot(311)
        plt.suptitle(title)
        plt.plot(x0,y0,'r',label=label)
        plt.ylabel('[%s]' % unit)
        plt.grid()
        plt.xlim(0.+3000*i,1000.+3000*i)
        plt.legend(loc=0)
        
        plt.subplot(312)
        plt.plot(x0,y0,'r',label=label)
        plt.ylabel('[%s]' % unit)
        plt.grid()
        plt.xlim(1000.+3000*i,2000.+3000*i)
        
        plt.subplot(313)
        plt.plot(x0,y0,'r',label=label)
        plt.xlabel('time [s]')
        plt.ylabel('[%s]' % unit)
        plt.grid()
        plt.xlim(2000.+3000*i,3000.+3000*i)
        plt.savefig(tofile + '_time%d.png' % i)
        plt.clf()





###########################################################
###########################################################



    
def make_plotMT(title,x0,y0,x1,y1,label1,label2,unit,tofile):
    plt.figure(figsize=(12,8))
    plt.axes([0.15,0.1,0.65,0.75])
    for i in range(0,5):

        plt.subplot(311)
        plt.title(title)
        plt.plot(x0,y0,'r',label=label1)
        plt.plot(x1,y1,'k',label=label2)
        plt.ylabel('[%s]' % unit)
        plt.grid()
        plt.xlim(0.+3000*i,1000.+3000*i)
        plt.legend(loc=0)
        
        plt.subplot(312)
        plt.plot(x0,y0,'r',label=label1)
        plt.plot(x1,y1,'k',label=label2)
        plt.ylabel('[%s]' % unit)
        plt.grid()
        plt.xlim(1000.+3000*i,2000.+3000*i)
        
        plt.subplot(313)
        plt.plot(x0,y0,'r',label=label1)
        plt.plot(x1,y1,'k',label=label2)
        plt.xlabel('time [s]')
        plt.ylabel('[%s]' % unit)
        plt.grid()
        plt.xlim(2000.+3000*i,3000.+3000*i)
        plt.savefig(tofile + '_time%d.png' % i)
        plt.clf()



###########################################################
###########################################################



def allstats(x,t):
    r"""timetools.allstats
        -----------------

        Returns "all" statistics similar as for model tests
        
        vak 06.05.2011

        -----------------

        Input:   x - time series 
                 t - time vector

        Output:  ? - ?

        -----------------"""

    x_mean    = x.mean()
    x_max     = x.max()
    x_min     = x.min()
    x_std     = x.std()
    x_nmax    = 0.
    x_nmin    = 0.
    x_nmcross = 0.
    x_maxpp   = 0.
    x_sgnpp   = 0.
    x_sgnmax  = 0.
    x_sgnmin  = 0.
    x_skew    = scipystats.skew(x)
    x_kurt    = scipystats.kurtosis(x)
    x_estmax  = 0.
    x_estmin  = 0.

    return x_mean,x_max,x_min,x_std,x_nmax,x_nmin,x_nmcross,x_maxpp,x_sgnpp,x_sgnmax,x_sgnmin,x_skew,x_kurt,x_estmax,x_estmin




###########################################################
###########################################################


def window(x,t,t0,duration,dt=None):
    r"""timetools.window
        -----------------
    
        Windows the time series x(t) between t0 <= t < t1
        where t1 = t0 + duration. 

        The function offers an optional resample to a 
        time step dt.

        tek 17.06.2010

        -----------------

        Input:   x - numpy time series array
        Output:  tuple(X(T), T) 
                  X(T) - windowed time series
                  T    - corresponding time vector
        -----------------"""

    ix =  np.nonzero(t > t0)
    x, t = x[ix], t[ix]
    ix =  np.nonzero(t < t0 + duration)
    x, t = x[ix], t[ix]
    if dt is not None: 
        rt = np.arange(t[0],t[-1],dt)
        return np.interp(rt, t, x), rt
    else:
        return x, t



import string
import numpy as np
import matplotlib.pyplot as plt

def floadtxt(filename, comments='#', skiprows=0):
    """ For reading Fortran column data with 'D' formatting
    """
    with open(filename, 'r') as fin:
        lines = [l for l in fin.readlines() if not l.strip().startswith(comments)][skiprows:]
        #lines = [l for l in lines if len(l.split())!=0]
        return np.array([list(map(float, l.replace('D', 'E').split())) for l in lines])

def read_probes(filename, skiprows=2, plotfile=None):
    """ 
    Reads a probe.dat file from HOS-NWT and returns the data
    Optionally plots to plotfile   
    """
    data = floadtxt(filename, skiprows=2).T
    time = data[0]
    probes = data[1:]
    fig, axs = plt.subplots(nrows=len(data)-1, ncols=1)
    nprobes = len(probes)
    if plotfile is not None:
        for i in range(nprobes):
            axs[i].plot(time, probes[i], label='P%d' % (i+1))
            axs[i].legend(loc=0)
            axs[i].set_ylabel('[m]')
            if i == nprobes - 1: axs[i].set_xlabel("time [s]")
        axs[0].set_title("Wave elevations - HOS-NWT Probes")
        plt.savefig(plotfile)
    return (time, probes)


#-------------------------------------------------------------------------#
#   TORSETHAUGEN (1996) WAVE SPECTRUM
#-------------------------------------------------------------------------#
#   modelled according to dnv c205 2010 p 110*
#   * wave spectrum is modified by a factor h1 and h2 to ensure that output
#   Hm0 is equal to the Hs
#
#   [S] = torsethaugen(Hs,Tp,ps,t_limit)
#
#   Hs:         significant wave height [m]
#   Tp:         spectral peak period [sec]
#   ps:         plot spectrum in figure
#   wave_spectrum:  matlab struct containing spectrum parameters
#   t_limit:    variable that can be left default the spectrum will then
#               be defined from 3 to 40 sec. [t_low t_high] can also be defined
#               as the boundary values for spectrum generation
#
#   af:         fetch length. set to 6.6
#
#   example:    torsethaugen(16.3,16.9,1)
#   example:    torsethaugen(16.3,16.9,0,[5 25])
#
#   10.09.2019: Converted to python to compare with SIMO
#
#-------------------------------------------------------------------------#


def torsethaugen1996(Hs, Tp, w=None, ps=False, t_limit=None):

    import numpy as np
    import matplotlib.pyplot as plt
    from scipy.special import gamma
    import sys

    # response variables
    wave_spectrum = {}
    g = 9.81

    # number of frequency components

    if t_limit is None:
        f_low = 1/40
        f_high = 1/3
    else:
        f_low = 1/t_limit[1]
        f_high = 1/t_limit[0]

    # fetch length
    af = 6.6

    # wind or swell distinction
    Tf = af * Hs ** (1/3.)
    if Tp <= Tf:
        sd = 'wind'
    elif Tp > Tf:
        sd = 'swell'

    # for all frequencies
    if w is not None:
        frequencies = w/2/np.pi + 0.0001
        ncomp = len(frequencies)
    else:
        ncomp = 200
        df = (f_high - f_low)/ncomp
        frequencies = np.arange(f_low, f_high, df)

    S = np.zeros((len(frequencies), 4))

    for fc, f in enumerate(frequencies):

        # common parameters
        N = 0.5 * np.sqrt(Hs) + 3.2

        #---------------------------------------------------------------------#
        # if wind dominated seas
        #---------------------------------------------------------------------#
        if sd == 'wind':

            rpw = 0.7 + 0.3 * np.exp(-(2*(Tf - Tp)/(Tf - 2*sqrt(Hs)))**2)

            # primary peak
            HS1 = rpw * Hs
            TP1 = Tp
            M = 4
            gma1 = 35. * (1 + 3.5*np.exp(-Hs)) * (2*np.pi/g * HS1/Tp**2)**0.857
            f2 = (2.2 * M**-3.3 + 0.57) * N**(0.53 -
                                              0.58*M**0.37) + 0.94 - 1.04*M**-1.9
            Agma1 = (4.1 * (N - 2*M**0.28 + 5.3)**(0.96 - 1.45*M**0.1)
                     * np.log(gma1)**f2 + 1.0) / gma1
            fn1 = f * TP1
            if fn1 < 1:
                sig = 0.07
            elif fn1 >= 1:
                sig = 0.09

            gmaF1 = gma1 ** np.exp(-1/(2*sig**2)*(fn1 - 1)**2)
            GMAS1 = fn1 ** -N * np.exp(-N/M * fn1**-M)
            G0 = (1/M * (N/M) ** (-(N-1)/M) * gamma((N-1)/M)) ** -1
            Sn1 = G0 * Agma1 * GMAS1 * gmaF1
            E1 = 1/16. * HS1**2 * TP1

            # secondary peak
            HS2 = np.sqrt(1-rpw**2)*Hs
            TP2 = Tf + 2.0
            M = 4
            gma2 = 1.
            f2 = (2.2 * M**-3.3 + 0.57) * N**(0.53 -
                                              0.58*M**0.37) + 0.94 - 1.04*M**-1.9
            Agma2 = (4.1 * (N - 2*M**0.28 + 5.3)**(0.96 - 1.45*M**0.1)
                     * np.log(gma2)**f2 + 1) / gma2
            gmaF2 = 1.
            fn2 = f * TP2
            GMAS2 = fn2 ** -N * np.exp(-N/M * fn2**-M)
            G0 = (1/M * (N/M) ** (-(N-1)/M) * gamma((N-1)/M)) ** -1
            Sn2 = G0 * Agma2 * GMAS2 * gmaF2
            E2 = 1/16. * HS2**2 * TP2

            #---------------------------------------------------------------------#
            # if swell dominated seas
            #---------------------------------------------------------------------#
        elif sd == 'swell':

            rps = 0.6 + 0.4 * np.exp(-((Tp - Tf)/(0.3 * (25 - Tf)))**2)

            # primary peak
            # disp('primary peak')
            HS1 = rps * Hs
            TP1 = Tp
            gma1 = 35. * (1 + 3.5*np.exp(-Hs)) * (2*np.pi/g * Hs/Tf **
                                                  2)**0.857 * (1. + 6*((Tp - Tf) / (25. - Tf)))
            M = 4.
            f2 = (2.2 * M**-3.3 + 0.57) * N**(0.53 -
                                              0.58*M**0.37) + 0.94 - 1.04*M**-1.9
            Agma1 = (4.1 * (N - 2*M**0.28 + 5.3)**(0.96 - 1.45*M**0.1)
                     * np.log(gma1)**f2 + 1.) / gma1
            fn1 = f * TP1
            if fn1 < 1:
                sig = 0.07
            elif fn1 >= 1:
                sig = 0.09

            gmaF1 = gma1 ** np.exp(-1/(2*sig**2)*(fn1 - 1)**2)
            GMAS1 = fn1 ** -N * np.exp(-N/M * fn1**-M)
            G0 = (1/M * (N/M) ** (-(N-1)/M) * gamma((N-1)/M)) ** -1
            Sn1 = G0 * Agma1 * GMAS1 * gmaF1
            E1 = 1/16. * HS1**2 * TP1

            # secondary peak
            # disp('secondary peak')
            HS2 = np.sqrt(1-rps**2)*Hs
            G0 = (1/M * (N/M) ** (-(N-1.)/M) * gamma((N-1.)/M)) ** -1
            s4 = np.max([0.01, 0.08 * (1. - np.exp(-1/3 * Hs))])
            tmp = ((16. * s4 * 0.4 ** N) / (G0 * HS2**2)) ** (-1./(N-1.))
            TP2 = np.max([2.5, tmp])
            gma2 = 1
            M = 4. * (1 - 0.7 * np.exp(-1/3 * Hs))
            f2 = (2.2 * M**-3.3 + 0.57) * N**(0.53 -
                                              0.58*M**0.37) + 0.94 - 1.04*M**-1.9
            Agma2 = (4.1 * (N - 2*M**0.28 + 5.3)**(0.96 - 1.45*M**0.1)
                     * np.log(gma2)**f2 + 1) / gma2
            gmaF2 = 1.
            fn2 = f * TP2
            GMAS2 = fn2 ** -N * np.exp(-N/M * fn2**-M)
            Sn2 = G0 * Agma2 * GMAS2 * gmaF2
            E2 = 1./16. * HS2**2 * TP2

        # spectrum frequency
        S[fc, 0] = f

        # primary spectrum
        S[fc, 1] = E1 * Sn1

        # secondary spectrum
        S[fc, 2] = E2 * Sn2

    # including factor to correct primary and secondary spectrum to input Hs
    f = S[:, 0]
    a1 = np.trapz(f**0 * S[:, 1], f)
    a2 = np.trapz(f**0 * S[:, 2], f)

    h1 = HS1**2 / (4*np.sqrt(a1))**2
    h2 = HS2**2 / (4*np.sqrt(a2))**2

    # corrected primary and secondary peak (to ensure Hm0 = Hs)
    S[:, 1] = S[:, 1] * h1
    S[:, 2] = S[:, 2] * h2
    S[:, 3] = S[:, 1] + S[:, 2]

    # calculating spectrum moments
    Sf = S[:, 3]
    m0 = np.trapz(f**0 * Sf, f)
    m1 = np.trapz(f**1 * Sf, f)
    m2 = np.trapz(f**2 * Sf, f)

    # calculating statistical parameters
    Hm0 = 4 * np.sqrt(m0)
    Tm01 = m0 / m1
    Tm02 = np.sqrt(m0/m2)

    wave_spectrum['spectrum'] = S
    wave_spectrum['m0'] = m0
    wave_spectrum['m1'] = m1
    wave_spectrum['m2'] = m2
    wave_spectrum['Hm0'] = Hm0
    wave_spectrum['Tm01'] = Tm01
    wave_spectrum['Tm02'] = Tm02

    # plot
    if ps is True:

        fig = plt.figure(figsize=(8, 12))
        ax1 = fig.add_subplot(4, 1, 1)
        ax1.plot(f, Sf)
        ax1.set_title('Torsethaugen 1996 wave spectrum Hs ' +
                      str(Hs) + ' m Tp ' + str(Tp) + ' s')
        ax1.set_xlabel('Hz [1/sec]')
        ax1.set_ylabel('S(f) m**2s')
        ax1.set_xlim(f_low, f_high)
        ax1.grid(True)

        ax2 = fig.add_subplot(4, 1, 2)
        ax2.plot(f*(2*np.pi), Sf/(2*np.pi))
        ax2.set_xlabel('w [rad/sec]')
        ax2.set_ylabel('S(w) m**2s/rad')
        ax2.set_xlim(2*np.pi*f_low, 2*np.pi*f_high)
        ax2.grid(True)

        ax3 = fig.add_subplot(4, 1, 3)
        ax3.plot(1./f, Sf)
        ax3.set_xlabel('T [sec]')
        ax3.set_ylabel('S(f) m**2s')
        ax3.set_xlim(1/f_high, 1/f_low)
        ax3.grid(True)

        ax4 = fig.add_subplot(4, 1, 4)
        ax4.plot(1./f, S[:, 1], label='primary peak')
        ax4.plot(1./f, S[:, 2], label='secondary peak')
        ax4.set_xlabel('T [sec]')
        ax4.set_ylabel('S(f) m**2s')
        ax4.legend(loc=0)
        ax4.set_xlim(1/f_high, 1/f_low)
        ax4.grid(True)
        plt.show()

    if w is None:
        return Sf/(2*np.pi), f*2*np.pi
    else:
        return Sf/(2*np.pi)


if __name__ == "__main__":
    import numpy as np
    import matplotlib.pyplot as plt
    w = np.linspace(0, 5, 254)
    #Sf, w = torsethaugen1996(4, 16)
    Sf = torsethaugen1996(4, 16, w)
    plt.plot(w, Sf)
    plt.show()



import setuptools
from numpy.distutils.core import setup
from numpy.distutils.misc_util import Configuration
from distutils.command.sdist import sdist
from sphinx.setup_command import BuildDoc
cmdclass = {'build_sphinx': BuildDoc}

PACKAGE_NAME = 'teklib'
config = Configuration(PACKAGE_NAME)

with open("README.md", "r") as fh:
    long_description = fh.read()

#setuptools.setup(
setup(
    **config.todict(),
    version="1.0.0",
    author="Tim Kendon",
    author_email="timothy.edward.kendon@gmail.com",
    description="Library of tools for the analysis of marine technology",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Timothy-Edward-Kendon/teklib",
    packages=setuptools.find_packages(where="src"),
    package_dir={'': 'src',},
    cmdclass=cmdclass,
    ext_modules = [],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=["numpy", "matplotlib", "scipy", "freesif"]
)


from .info import __doc__
from . import administration
from . import application
from . import wave_theory
from . import signal_analysis
from . import miscellaneous
from . import software

__version__ = "1.0.0"




#!/home/draugen/a/tken/Python254/bin/python

#
# Usage: General postprocessing tool for WAMIT.out file(s)
#

# Read in modules
import optparse
import sys
import os
from numpy import array, zeros, eye, e, pi, all, ones
import matplotlib.ticker as ticker
from pylab import *
from ..administration.tools import runprocess, multipleReplace, find_occurrences

import subprocess as sp
from datetime import date

class Usage(Exception):
    """ Standard Error Handler """
    def __init__(self, msg):
        self.msg = msg

#
# Helper functions and class
#

def phase0(ph, mode):
    """ With symmetries X = 0 and Y = 0 apply phase transformation
    
           Wdir:  0
           Surge: 0.  Sway:  0.  Heave: 0.
           Roll:  pi  Pitch: pi  Yaw:   pi

    This is a combination of 2 transformations:

    1. Wamit 0->90 to Wamit 90->180
 
            Wdir:  180-theta
            Surge: pi  Sway:  0.  Heave: 0.
            Roll:  0.  Pitch: pi  Yaw:   pi

    2. Wamit (90->180) to Mossi (0->90)

            Wdir:  180-theta
            Surge: pi  Sway:  0.  Heave: 0.
            Roll:  pi  Pitch: 0.  Yaw:   0.
           
    N.B. The latter transformation is opposite to that
    given in RIFLEX-C appendix 7A and WAMOF theory manuals.
    However it is specified as above in the WAM2MOS manual
    written by Knut Mo. The RIFLEX and WAMOF manuals appear
    to have a phase shift of pi for all modes. Having spoken
    Knut the reason for the extra pi shift in WAM2MOS
    is because the wave has a phase shift of pi. 

    Therefore all transfer functions have a phase shift of pi. 
    The WAMOF code has a slightly quirkier specification than
    WAMIT, however I found that the asymptotic limit of WAMIT
    and WAMOF were the same :/. Anyway I am happy with Knuts 
    logic! Further I have checked that his code exports as 
    above. 
    
    It also seems that the wave drift are opposite to that
    above :/ This is possibly due to an extra pi change as
    the wave drift has 2 waves hence pi of the second needs
    considering. :/ not sure on this logic. 
    """
    
    Surge, Sway, Heave, Roll, Pitch, Yaw = 0, 1, 2, 3, 4, 5
    if mode == Surge: phase = ph + 0.
    if mode == Sway:  phase = ph + 0.
    if mode == Heave: phase = ph + 0.
    if mode == Roll:  phase = ph + 180.
    if mode == Pitch: phase = ph + 180.
    if mode == Yaw:   phase = ph + 180.  
    if phase > 360.: phase = phase - 360.
    return phase

def phase1(ph, mode):
    """ 
        With only Y=0 plane symmetry or no-symmetry apply phase transformation
        
            Wdir:  180-theta
            Surge: pi  Sway:  0.  Heave: 0.
            Roll:  pi  Pitch: 0.  Yaw:   0. """
    Surge, Sway, Heave, Roll, Pitch, Yaw = 0, 1, 2, 3, 4, 5      
    if mode == Surge: phase = ph + 180.
    if mode == Sway:  phase = ph + 0.
    if mode == Heave: phase = ph + 0.
    if mode == Roll:  phase = ph + 180.
    if mode == Pitch: phase = ph + 0.
    if mode == Yaw:   phase = ph + 0.        
    if phase > 360.: phase = phase - 360.
    return phase

def dirch0(d):
    return d
    
def dirch1(d):
    d2 = 180. - d 
    if d2 > 360.: d2 = d2 - 360.
    return d2

def pha(z):
    return math.atan2(z.imag,z.real)*180./math.pi

                
def find_locations(string,data):
    return [loc for loc, line in enumerate(data) if string in line]


def flatten(iterable):
  it = iter(iterable)
  for e in it:
    if isinstance(e, (list, tuple)):
      for f in flatten(e):
        yield f
    else:
      yield e



def main():
    """ Main program for wamitlib.py """
    parser = optparse.OptionParser(
             description= 'General postprocessor for mimosa',
             prog       = 'mimosa.py',
             version    = 'mimosa.py 0.1',
             usage      = '%prog [options]')

    # define options
    parser.add_option('-w', dest="wamitfile", help="Wamit to Mossi Conversion")
    parser.add_option('-a', dest="amplitude", help="Incl. WAMOF heave (req. files must exist!)")
    parser.add_option('-c', action="store_true", help="Compare <mosfile 1> <mosfile 2>")
    parser.add_option('-s', dest="skiplist",  help="Smooth over id list of bad WD Coeffs")    

    options, arguments = parser.parse_args()

    # MENU MENU MENU MENU MENU MENU :) :/ 8(
    if options.wamitfile is not None: 
        return(wam2mos(options.wamitfile,options.amplitude,options.skiplist))
    if (options.c) and len(arguments) is not 2: 
        print("Need 2 arguments for this option: %d provided" % len(arguments))
        print(parser.print_help()); return
    if (options.c): return(compare_mossies(arguments))

    print(parser.print_help()); return
    

def wam2mos(wamitfile, amplitude, skiplist):
    """ Converts wamit files to mossi files """
    if amplitude is not None:
        data = None #read_wamit2(wamitfile,amplitude,skiplist)
    else:
        data = None #read_wamit(wamitfile, skiplist)
    # Open the Mossi FILE
    mossi = open(wamitfile.strip(".out") + (".mos"),"w")
    mossi.write("'File Generated using wam2mos.py'\n")
    mossi.write("'WAMIT FILE: %s, Date: %s, User ID: TEK'\n" % (wamitfile,date.today().isoformat()))
    # Output DATA Group 1
    mossi.write("'DATA GROUP 1 - MASS, ADDED MASS AND HYDRODYNAMIC STIFFNESS'\n")
    mossi.write("11000 Dry Mass Matrix\n")
    mossi.write("11100 0.0 0.0 0.0\n")    
    # Output Mass Matrix
    M = data["Mass"]
    M[4,0] = -M[4,0]; M[0,4] = -M[0,4]
    M[3,1] = -M[3,1]; M[1,3] = -M[1,3] 
    M[3,2] = -M[3,2]; M[2,3] = -M[2,3] 
    M[5,0] = -M[5,0]; M[0,5] = -M[0,5] 
    M[3,4] = -M[3,4]; M[4,3] = -M[4,3] 
    M[5,3] = -M[5,3]; M[3,5] = -M[3,5]
    #if M.all() == 0. : print "Mass matrix not found"
    for i in range(0,6): mossi.write("1110%d%12.4e%12.4e%12.4e%12.4e%12.4e%12.4e\n" 
              % (i+1, M[i,0], M[i,1], M[i,2], M[i,3], M[i,4], M[i,5]))
    # Output Added Mass Data
    A = data["AddedMass"]; T     = data["Periods"]
    mossi.write("11200 Added Mass Matrix (at T = %.1f secs)\n" % max(T))
    AM = A[T.argmax(),:,:]
    AM[4,0] = -AM[4,0]; AM[0,4] = -AM[0,4]
    AM[3,1] = -AM[3,1]; AM[1,3] = -AM[1,3] 
    AM[0,2] = -AM[0,2]; AM[2,0] = -AM[2,0] 
    AM[3,2] = -AM[3,2]; AM[2,3] = -AM[2,3] 
    AM[5,0] = -AM[5,0]; AM[0,5] = -AM[0,5] 
    AM[3,4] = -AM[3,4]; AM[4,3] = -AM[4,3] 
    AM[5,3] = -AM[5,3]; AM[3,5] = -AM[3,5]
    for i in range(0,6): mossi.write("1120%d%12.4e%12.4e%12.4e%12.4e%12.4e%12.4e\n" 
              % (i+1, AM[i,0], AM[i,1], AM[i,2], AM[i,3], AM[i,4], AM[i,5]))
    # Output Gravitational and Hydrostatic Stiffness Matrix
    K = data["KMatrix"]
    mossi.write("12000 Hydrostatic Stiffness Data\n")
    mossi.write("12100%12.1f%12.1f\n" % (0., 0.))    
    mossi.write("12101%12.3e%12.3e%12.3e\n" % (K[2,2],K[2,3],K[2,4]))
    mossi.write("12102%12.3e%12.3e%12.3e\n" % (K[3,2],K[3,3],K[3,4]))
    mossi.write("12103%12.3e%12.3e%12.3e\n" % (K[4,2],K[4,3],K[4,4]))
    # Output Wind and Current Coefficients
    mossi.write("'DATA GROUP 2 - CURRENT AND WIND FORCE COEFFICIENTS'\n")
    mossi.write("20000 Coefficients from Wind Tunnel Tests\n")
    mossi.write("'Quadratic and Linear Current Coefficients'\n")
    #mossi.write("'# of Quadratic and Linear Current Coefficients\n")
    mossi.write("23100  37 37\n")
    #mossi.write("'Quadratic Current Coefficients\n")
    for i in range(0,37):
        mossi.write("231%02d%12.1f%12.3e%12.3e%12.3e%12.3e%12.3e%12.3e\n" % \
                            (i+1, i*10., 0., 0., 0., 0., 0., 0.))
    #mossi.write("'Linear Current Coefficients\n")
    for i in range(0,37):
        mossi.write("232%02d%12.1f%12.3e%12.3e%12.3e%12.3e%12.3e%12.3e\n" % \
                            (i+1, i*10., 0., 0., 0., 0., 0., 0.))
    # Output Quadratic Damping Force
    mossi.write("'Quadratic Damping Coefficients'\n")
    mossi.write("23500 0.0 0.0 \n")
    for i in range(1,7):
        mossi.write("2350%d%12.3e\n" % (i,0.))
                
    mossi.write("'Linear Damping Coefficients'\n")
    mossi.write("23600 0.0 0.0 \n")
    for i in range(1,7):
        mossi.write("2360%d%12.3e%12.3e%12.3e%12.3e%12.3e%12.3e\n" % \
                            (i, 0., 0., 0., 0., 0., 0.))              
    # Output Wind force coefficients
    mossi.write("'Quadratic Wind Force Coefficients'\n")
    #mossi.write("'# of Wind Force Coefficients\n")
    mossi.write("24100  37\n")
    for i in range(0,37):
        mossi.write("241%02d%12.1f%12.3e%12.3e%12.3e%12.3e%12.3e%12.3e\n" % \
                            (i+1, i*10., 0., 0., 0., 0., 0., 0.))
    # Need to sort frequencies into ascending order
    T     = data["Periods"]; w   = 2*pi/T; Dir = data["Headings"]
    Drift = data["Drift_P"]; RAO = data["R.A.O"]; 

    # temporarily replace Drift_P with Drift_M for surge direction!
    #Drift2 = data["Drift_M"]; Drift[:,:,0] = Drift2[:,:,0]*1.0
    
    # temporarily scale surge drift by 15%
    #Drift[:,:,0] = Drift[:,:,0]*1.15

    sym = data["Symmetry"];  wavek = data["Wavenumbers"]
    if (w[0] > w[1]): 
        w = w[::-1] # Reverse frequencies and reverse array rows
        wavek = wavek[::-1] # Reverse wave numbers
        for m in range(0,6): # Note numpy arrays are referenced
            Drift[:,:,m] = Drift[::-1,:,m].copy() #could use *1
            RAO[:,:,m]   = RAO[::-1,:,m].copy()   #could use *1
    # Need drift coefficients at least between (0,180)    
    if sym =="X&Y":
        Dir2   = zeros(2*len(Dir)-1) #-1 as they share 90
        Drift2 = zeros((len(w),len(Dir2),6))
        for i, d in enumerate(Dir):
            Dir2[i]  = d; Dir2[-(i+1)] = 180.-d
        for i, f in enumerate(w):
            for j, d in enumerate(Dir):
                for m in range(0,6):
                    Drift2[i,  j   ,m] = Drift[i,j,m]
                    Drift2[i,-(j+1),m] = Drift[i,j,m]
                    # Need a pi change for Surge, Pitch and Yaw
                    if m == 0 or m > 3: Drift2[i,-(j+1),m] = -Drift[i,j,m]
    # Set transformations for mossiconversion
    dirchange   = {0 : dirch0, 1 : dirch1}
    phasechange = {0 : phase0, 1 : phase1}
    print("%s symmetry plane(s) found" % sym)
    if sym == "X&Y": itrans = 0; isys = 2
    if sym == "Y=0": itrans = 1; isys = 1
    if sym == None:  itrans = 1; isys = 0
    if sym == "X=0":
        print("Not implemented")
        sys.exit(-1)
    mossi.write("'DATA GROUP 3 - TRANSFER FUNCTION COEFFICIENTS'\n")                
    mossi.write("30000 Motion Transfer Functions\n")                
    mossi.write("30100 %d" % len(w))
    # Output Frequencies
    j = 1 
    for i, f in enumerate(w):
        if i % 5 == 0:
            mossi.write("\n301%02d" % j)
            j = j + 1
        mossi.write("%12.4f" % f)
        if (i+1) == len(w):
            mossi.write("\n")
    # Output directions
    if sym == "X&Y": ind = list(range(0,len(Dir)))
    else:            ind = list(range(len(Dir)-1,-1,-1))
    j = 1 
    if Dir[0] > Dir[1]: print("Wamit directions descending :/")
    mossi.write("30200 %d %d" % (len(Dir), isys))
    for k, i in enumerate(ind): 
        if k % 5 == 0:
            mossi.write("\n302%02d" % j)
            j = j + 1
        mossi.write("%12.1f" % (dirchange[itrans](Dir[i])))
        if (k+1) == len(Dir):
            if k % 5 is not 0:
                mossi.write("\n")
    # Output Water Depth
    mossi.write("30300 %s\n" % data["Depth"])
    # R.A.Os
    k = 30
    for d in ind:
        k = k + 1
        if k == 40: k = 31
        for m, mode in enumerate(["Surge","Sway","Heave"]):
            for i in range(0,len(w)):
                mossi.write("%02d%d%02d%12.3e%12.3e\n" % 
                                (k, m+1, i+1, abs(RAO[i,d,m]), 
                                 phasechange[itrans](pha(RAO[i,d,m]),m)))
                    
        for n, mode in enumerate(["Roll","Pitch","Yaw"]):
            m = n + 3
            for i in range(0,len(w)):
                mossi.write("%02d%d%02d%12.3e%12.3e\n" % 
                                (k, m+1, i+1, abs(RAO[i,d,m])/wavek[i], 
                                 phasechange[itrans](pha(RAO[i,d,m]),m)))


    # Output Wave drift
    if sym == "X&Y":
        Dir = Dir2; Drift = Drift2
    mossi.write("'DATA GROUP 4 - WAVE DRIFT FORCE COEFFICIENTS'\n")
    mossi.write("40000 Mean Wave Drift Coefficients (no scaling)\n")
    mossi.write("40100 %d %d Aranha 50." % (len(w),len(Dir)))
    # Output frequencies
    k = 1
    for i, f in enumerate(w):
        if i % 5 == 0:
            mossi.write("\n402%02d" % k)
            k = k + 1
        mossi.write("%12.4f" % f)     
    # directions (note that for wave drift, the X&Y=0 case
    # has been expanded to a Y=0 case, hence Dir[::-1].
    # Also the itrans and isym need changing to Y=0 case
    if sym == "X&Y": itrans = 1; isys = 1
    k = 1
    for i, d in enumerate(Dir[::-1]):
        if i % 5 == 0:
            mossi.write("\n403%02d" % k)
            k = k + 1
        mossi.write("%12.1f" % (dirchange[itrans](d)))
        if (i+1) == len(Dir):
            if i % 5 is not 0:
                mossi.write("\n")
    # W.D. Coefficients
    k = 0
    for d in range(len(Dir)-1,-1,-1):
        k = k + 1
        if k == 10: k = 1
        for i in range(0,len(w)):
            p0 = pha(Drift[i,d,0])
            p1 = pha(Drift[i,d,1])
            p2 = pha(Drift[i,d,2])
            p3 = pha(Drift[i,d,3])
            p4 = pha(Drift[i,d,4])
            p5 = pha(Drift[i,d,5])
            sign = [1,1,1,1,1,1]
            # apply an implicit pi phase shift (as wave cancels)
            for m, p in enumerate([p0,p1,p2,p3,p4,p5]):
                #if 170. < abs(phasechange[itrans](p,m)) < 190.: 
                if  abs(phasechange[itrans](p,m))> 340. or \
                        abs(phasechange[itrans](p,m))<  20.: 
                    sign[m] = -1
                #
            mossi.write("41%d%02d" % (k, i+1)) 
            mossi.write("%12.3e%12.3e%12.3e%12.3e%12.3e%12.3e\n" % 
                        (abs(Drift[i,d,0])*sign[0],
                         abs(Drift[i,d,1])*sign[1],
                         abs(Drift[i,d,2])*sign[2],
                         abs(Drift[i,d,3])*sign[3],
                         abs(Drift[i,d,4])*sign[4],
                         abs(Drift[i,d,5])*sign[5]))

def find_mossi_id(data,id):
    for l, line in enumerate(data):
        if line.startswith(id): return l
    return None

def read_mossi(mos):
    print("Scanning %s" % mos)
    data = open(mos, "r").readlines()
    loc  = find_mossi_id(data,"40100")
    id, nf, nd = [float(i) for i in data[loc].strip().split()[:3]]
    nf, nd, cl = int(nf), int(nd), loc + 1
    nflines, ndlines = (nf-1) / 5 + 1, (nd-1)/5 + 1
    freqblock = data[cl:cl+nflines]
    headblock = data[cl+nflines:cl+nflines+ndlines]
    L = [line.split()[1:] for line in freqblock]
    freqs = array([float(i) for i in list(flatten(L))])
    L = [line.split()[1:] for line in headblock]
    heads = array([float(i) for i in list(flatten(L))])
    cl = cl+nflines+ndlines; DOF = len(data[cl].strip().split()[1:])
    print("Reading %d-DOF Wave Drift Coefficients" % DOF) 
    L = [line.split()[1:] for line in data[cl:cl+nf*nd]]
    WDCoeffs = array([float(i) for i in list(flatten(L))]).reshape((nd,nf,DOF))
    return {"Frequencies": freqs, "Headings": heads, "WDCoeffs": WDCoeffs, "DOF": DOF}


def compare_mossies(mossifiles):
    mos1, mos2 = mossifiles
    VD1 = read_mossi(mos1)
    VD2 = read_mossi(mos2)
    h1, f1, wd1 = VD1["Headings"], VD1["Frequencies"], VD1["WDCoeffs"]
    h2, f2, wd2 = VD2["Headings"], VD2["Frequencies"], VD2["WDCoeffs"]
    figure(figsize=(11,6))
    axes([0.15,0.1,0.65,0.75])
    ax = gca()
    ax.semilogx()
    ax.xaxis.set_major_formatter(ticker.FormatStrFormatter("%.f"))
    ax.xaxis.set_minor_locator(ticker.MultipleLocator(1))
    #ax.set_xticks([1,2,3,4,5,6,7,8,9,10,20,30,40,50,60,70])
    ax.set_xticks([2,3,4,5,6,7,8,9,10,20,30,40,50,60,70])
    ax.xaxis.grid(True, which='minor')
    ax.xaxis.grid(True, which='major', color='k', linestyle='--')
    ax.yaxis.grid(True, which='major')
    title("Mean Drift Force Coefficient in Surge")# + mode) 
    plot(2*pi/f2,wd2[0,:,0],marker="D")
    plot(2*pi/f1,wd1[0,:,0],marker="^")
    grid(True)
    xlabel("Period [s]")
    #ylabel(ftype[m][1] + 
    #       " / Amplitude Squared of Incident Wave kN/m^2")# + ftype[m][0])
    #xlim(0.,xmax)
    T1, T2 = 2*pi/f1, 2*pi/f2
    xlim(2.,1.5*2*pi/f2[0])
    ind = nonzero(T2>=2)[0]
    #ylim(1.5*(wd1[ix_(ind),:,m].real).min()/1000.,
    #     1.5*(wd2[ix_(ind),:,m].real).max()/1000.)
    #savefig("figures/"+dt[1]+"_"+mode+".png")
    #savefig("figures/"+dt[1]+"_"+mode+".eps")
    show()


#
#---- EXPORT REPORT FILE FOR POSITIONING SYSTEM
#


POSITIONINGMACRO = \
""" /            ' Long-term simulation ? (N)
 @ wait off
 3               ' Number of degrees of freedom (3 or 6)
 REPORT.DAT
 /               ' Append data to file if it already exists ? (N)
 Output positioning system
 for conversion to riflex inpmod
 READ SYSTEM                   ' SYSTEM
 MOORING SYSTEM                ' SYSTEM READ
 xxxmim
 Return                        ' SYSTEM READ
   MOORING SYSTEM COMPUTATION  ' SYSTEM
 /                     ' Result to file ? (N)
   SYSTEM                      ' MOORING SYSTEM COMPUTATIONS
 PRINT/DRAW/STORE SYSTEM       ' SYSTEM
 POSITIONING SYSTEM PRINTOUT   ' PRINT/DRAW/STORE POSITIONING SYSTEM DATA
 y                     ' Results to file ? (N)
 LINE CHARACTERISTICS PRINTOUT ' PRINT/DRAW/STORE POSITIONING SYSTEM DATA
 y                     ' Results to file ? (N)
 Return                        ' PRINT/DRAW/STORE POSITIONING SYSTEM DATA
 Terminate                     ' SYSTEM
"""

def returnpositioningdata(mimfile):
    """ Returns tuple (linedata, linecharacteristics)
          
           linedata is a dictionary of linedata :)

           linecharacteristics is a list containing:

               1. dictionarys where the line characteristic is used
               2. or the following strings
                     'Line characteristic not used (removed)'
                     'Table is direct input'

    """

    # RUN MIMOSA AND EXPORT A REPORT FILE
    #-----------------------------------------
    MACRO = multipleReplace(POSITIONINGMACRO, 
                            {'xxxmim':mimfile}) 
    with open('MIMOSA.INP','w') as macroFile:
        macroFile.write(MACRO)
    #runprocess("C:/Appl/BatchScripts/mimosa.bat") # generates REPORT file
    os.system("C:/Appl/BatchScripts/mimosa.bat") # generates REPORT file
    with open('REPORT.DAT','r') as repFile:
        reportlines = repFile.readlines()
    with open(mimfile, 'r') as mimFile: 
        mimlines    = mimFile.readlines()
    numberofmooringlines = len(find_occurrences('LINE DATA', mimlines))
    # GET FAIRLEAD INFORMATION
    #-----------------------------------------
    skiplines = 7
    location = find_occurrences('LINE DATA', reportlines, 
                    specificOccurrenceWanted= 1) + skiplines
    informationonfairleads = reportlines[location:location+numberofmooringlines]
    xyfairlead = np.zeros((numberofmooringlines, 2))
    linechar   = np.zeros(numberofmooringlines,np.int)
    for i, line in enumerate(informationonfairleads): 
        words = line.split()
        xyfairlead[i,:] = np.r_[[float(word) for word in words[-2:]]]
        linechar[i] = int(words[1])

    #----------------------------------------------------------
    location = find_occurrences('Heading (rel. North)', reportlines,
                               specificOccurrenceWanted =1)
    vesselheading = reportlines[location].split(':')[1]
    vesselheading = float(vesselheading.strip().strip('deg.'))
    #----------------------------------------------------------
    location = find_occurrences('X1  (Northwards)', reportlines,
                               specificOccurrenceWanted =1)
    vesselxoffset = reportlines[location].split(':')[1]
    vesselxoffset = float(vesselxoffset.strip().strip('m'))
    #----------------------------------------------------------
    location = find_occurrences('X2  (Eastwards)', reportlines,
                               specificOccurrenceWanted =1)
    vesselyoffset = reportlines[location].split(':')[1]
    vesselyoffset = float(vesselyoffset.strip().strip('m'))
    

    
    # GET ANCHOR COORDINATES, TENSION ETC ...
    #-----------------------------------------
    skiplines = 6
    location = find_occurrences('LINE INITIALIZING DATA', reportlines, 
                               specificOccurrenceWanted = 1) + skiplines
    informationonanchor = reportlines[location:location+numberofmooringlines]
    xyanchor   = np.zeros((numberofmooringlines, 2))
    heading    = np.zeros(numberofmooringlines)
    tension    = np.zeros(numberofmooringlines)
    horizontal = np.zeros(numberofmooringlines)
    winchlength= np.zeros(numberofmooringlines)
    for i, line in enumerate(informationonanchor):
        words = line.replace('*)','').split()
        xyanchor[i,:] = np.r_[[float(word) for word in words[-3:-1]]]
        heading[i], tension[i], horizontal[i] = \
            [float(word) for word in words[1:-3]]
        winchlength[i] = float(words[-1])

    # GET LINE CHARACTERISTICS (FROM MIM FILE)
    #-----------------------------------------
    locations = find_occurrences('LINE CHARACTERISTIC NUMBER', reportlines)
    numberoflinechar = len(locations)
    linecharacteristics = [None]*numberoflinechar
    locations.append(len(reportlines))
    for startloc, endloc in zip(locations[:-1],locations[1:]):
        ok = True
        linecharblockofdata = reportlines[startloc:endloc] 
        #---------------------------------------------------------------
        for line in linecharblockofdata:
            if 'LINE CHARACTERISTIC NUMBER' in line: 
                lchar = int(line.split(':')[1])
            if 'Table is direct input' in line: 
                linecharacteristics[lchar-1] = 'Table is direct input'
                ok = False
        if lchar not in linechar: 
            linecharacteristics[lchar-1] = 'Line characteristic not used (removed)'
            ok = False
        if not ok: continue # = cycle: poor terminology borrowed from C
        #---------------------------------------------------------------
        for line in linecharblockofdata:
            if 'LINE CHARACTERISTIC NUMBER' in line: 
                lchar = int(line.split(':')[1])
            if 'fairlead' in line: zfairlead = float(line.split(':')[1])
            if 'anchor'   in line: zanchor   = float(line.split(':')[1])
            if 'segments' in line: nsegments = int(line.split(':')[1])
        #---------------------------------------------------------------
        skiplines = 1
        location  = find_occurrences('elements', linecharblockofdata,
                                    specificOccurrenceWanted = 1) + skiplines
        subblock = linecharblockofdata[location:location+nsegments]
        nelseg   = np.zeros(nsegments, int)
        ibuoy    = np.zeros(nsegments, int)
        slength  = np.zeros(nsegments)        
        for i, line in enumerate(subblock): 
            words = line.split()
            nelseg[i], ibuoy[i] = int(words[2]), int(words[3])
            slength[i] = float(words[4]) 
        #---------------------------------------------------------------
        skiplines = 2
        location  = find_occurrences('elasticity', linecharblockofdata,
                                    specificOccurrenceWanted = 1) + skiplines
        subblock = linecharblockofdata[location:location+nsegments]
        diameter, emod, emfact, uwiw, watfac, cdn, cdl = \
            [np.zeros(nsegments) for i in range(0,7)]
        for i, line in enumerate(subblock):
            diameter[i], emod[i], emfact[i], uwiw[i], \
                watfac[i], cdn[i], cdl[i] = \
                [float(word) for word in line.split()[1:]]
        #---------------------------------------------------------------
        # pack into a dictionary 
        linecharacteristics[lchar-1] = \
            {'lchar'    : lchar,
             'nelseg'   : nelseg,
             'zfairlead': zfairlead,
             'zanchor'  : zanchor,
             'nsegments': nsegments,
             'slength'  : slength,
             'ibuoy'    : ibuoy,
             'slength'  : slength,
             'diameter' : diameter,
             'emod'     : emod,
             'emfact'   : emfact,
             'uwiw'     : uwiw,
             'watfac'   : watfac,
             'cdn'      : cdn,
             'cdl'      : cdl}
    
    #---------------------------------------------------------------
    linedata = \
        {'linechar'   : linechar,
         'xyfairlead' : xyfairlead,
         'xyanchor'   : xyanchor, 
         'heading'    : heading, 
         'tension'    : tension, 
         'horizontal' : horizontal,
         'winchlength': winchlength,
         'vesselheading': vesselheading,
         'vesselxoffset': vesselxoffset,
         'vesselyoffset': vesselyoffset}

    # GET BUOY DATA
    #-----------------------------------------
    locations = find_occurrences('ZBUOY (m)', reportlines)
    numberofbuoys = len(locations)
    buoyinfo      = [None]*numberofbuoys
    for i, loc in enumerate(locations): 
        zbuoy, fbuoy = reportlines[loc+1].split()
        buoyinfo[i] = (float(zbuoy), float(fbuoy))
    #---------------------------------------------------------------
    return (linedata, linecharacteristics, buoyinfo)


#-----------------------------------------------------------

POSITIONINGMACRO2 = \
""" /            ' Long-term simulation ? (N)
 @ wait off
 3               ' Number of degrees of freedom (3 or 6)
 REPORT.DAT
 /               ' Append data to file if it already exists ? (N)
 Output positioning system
 for conversion to riflex inpmod
 READ SYSTEM                   ' SYSTEM
 MOORING SYSTEM                ' SYSTEM READ
 xxxmim
 Return                        ' SYSTEM READ
   MOORING SYSTEM COMPUTATION  ' SYSTEM
 /                     ' Result to file ? (N)
 EQUILIBRIUM POSITION          ' MOORING SYSTEM COMPUTATIONS
 y                     ' Move vessel to equilibrium position? (Y)
 n                     ' Results to file ? (N)
 n                     ' Result to file ? (N)
   SYSTEM                      ' MOORING SYSTEM COMPUTATIONS
 PRINT/DRAW/STORE SYSTEM       ' SYSTEM
 POSITIONING SYSTEM PRINTOUT   ' PRINT/DRAW/STORE POSITIONING SYSTEM DATA
 y                     ' Results to file ? (N)
 LINE CHARACTERISTICS PRINTOUT ' PRINT/DRAW/STORE POSITIONING SYSTEM DATA
 y                     ' Results to file ? (N)
 Return                        ' PRINT/DRAW/STORE POSITIONING SYSTEM DATA
 Terminate                     ' SYSTEM
"""



def returnpositioningdata2(mimfile):
    """ Returns tuple (linedata, linecharacteristics)
          
           linedata is a dictionary of linedata :)

           linecharacteristics is a list containing:

               1. dictionarys where the line characteristic is used
               2. or the following strings
                     'Line characteristic not used (removed)'
                     'Table is direct input'

    """

    # RUN MIMOSA AND EXPORT A REPORT FILE
    #-----------------------------------------
    MACRO = multipleReplace(POSITIONINGMACRO2, 
                            {'xxxmim':mimfile}) 
    open('RUN.MAC','w').write(MACRO)
    runprocess("mimosa < RUN.MAC") # generates REPORT file
    reportlines = open('REPORT.DAT','r').readlines()
    mimlines    = open(mimfile, 'r').readlines()
    numberofmooringlines = len(find_occurrences('LINE DATA', mimlines))

    # GET FAIRLEAD INFORMATION
    #-----------------------------------------
    skiplines = 7
    location = find_occurrences('LINE DATA', reportlines, 
                                specificOccurrenceWanted = 1) + skiplines
    informationonfairleads = reportlines[location:location+numberofmooringlines]
    xyfairlead = np.zeros((numberofmooringlines, 2))
    linechar   = np.zeros(numberofmooringlines,np.int)
    for i, line in enumerate(informationonfairleads): 
        words = line.split()
        xyfairlead[i,:] = np.r_[[float(word) for word in words[-2:]]]
        linechar[i] = int(words[1])

    #----------------------------------------------------------
    location = find_occurrences('Heading (rel. North)', reportlines,
                               specificOccurrenceWanted =1)
    vesselheading = reportlines[location].split(':')[1]
    vesselheading = float(vesselheading.strip().strip('deg.'))
    #----------------------------------------------------------
    location = find_occurrences('X1  (Northwards)', reportlines,
                               specificOccurrenceWanted =1)
    vesselxoffset = reportlines[location].split(':')[1]
    vesselxoffset = float(vesselxoffset.strip().strip('m'))
    #----------------------------------------------------------
    location = find_occurrences('X2  (Eastwards)', reportlines,
                               specificOccurrenceWanted =1)
    vesselyoffset = reportlines[location].split(':')[1]
    vesselyoffset = float(vesselyoffset.strip().strip('m'))
    

    
    # GET ANCHOR COORDINATES, TENSION ETC ...
    #-----------------------------------------
    skiplines = 6
    location = find_occurrences('LINE INITIALIZING DATA', reportlines, 
                               specificOccurrenceWanted = 1) + skiplines
    informationonanchor = reportlines[location:location+numberofmooringlines]
    xyanchor   = np.zeros((numberofmooringlines, 2))
    heading    = np.zeros(numberofmooringlines)
    tension    = np.zeros(numberofmooringlines)
    horizontal = np.zeros(numberofmooringlines)
    winchlength= np.zeros(numberofmooringlines)
    for i, line in enumerate(informationonanchor):
        words = line.replace('*)','').split()
        xyanchor[i,:] = np.r_[[float(word) for word in words[-3:-1]]]
        heading[i], tension[i], horizontal[i] = \
            [float(word) for word in words[1:-3]]
        winchlength[i] = float(words[-1])

    # GET LINE CHARACTERISTICS (FROM MIM FILE)
    #-----------------------------------------
    locations = find_occurrences('LINE CHARACTERISTIC NUMBER', reportlines)
    numberoflinechar = len(locations)
    linecharacteristics = [None]*numberoflinechar
    locations.append(len(reportlines))
    for startloc, endloc in zip(locations[:-1],locations[1:]):
        ok = True
        linecharblockofdata = reportlines[startloc:endloc] 
        #---------------------------------------------------------------
        for line in linecharblockofdata:
            if 'LINE CHARACTERISTIC NUMBER' in line: 
                lchar = int(line.split(':')[1])
            if 'Table is direct input' in line: 
                linecharacteristics[lchar-1] = 'Table is direct input'
                ok = False
        if lchar not in linechar: 
            linecharacteristics[lchar-1] = 'Line characteristic not used (removed)'
            ok = False
        if not ok: continue # = cycle: poor terminology borrowed from C
        #---------------------------------------------------------------
        for line in linecharblockofdata:
            if 'LINE CHARACTERISTIC NUMBER' in line: 
                lchar = int(line.split(':')[1])
            if 'fairlead' in line: zfairlead = float(line.split(':')[1])
            if 'anchor'   in line: zanchor   = float(line.split(':')[1])
            if 'segments' in line: nsegments = int(line.split(':')[1])
        #---------------------------------------------------------------
        skiplines = 1
        location  = find_occurrences('elements', linecharblockofdata,
                                    specificOccurrenceWanted = 1) + skiplines
        subblock = linecharblockofdata[location:location+nsegments]
        nelseg   = np.zeros(nsegments, int)
        ibuoy    = np.zeros(nsegments, int)
        slength  = np.zeros(nsegments)        
        for i, line in enumerate(subblock): 
            words = line.split()
            nelseg[i], ibuoy[i] = int(words[2]), int(words[3])
            slength[i] = float(words[4]) 
        #---------------------------------------------------------------
        skiplines = 2
        location  = find_occurrences('elasticity', linecharblockofdata,
                                    specificOccurrenceWanted = 1) + skiplines
        subblock = linecharblockofdata[location:location+nsegments]
        diameter, emod, emfact, uwiw, watfac, cdn, cdl = \
            [np.zeros(nsegments) for i in range(0,7)]
        for i, line in enumerate(subblock):
            diameter[i], emod[i], emfact[i], uwiw[i], \
                watfac[i], cdn[i], cdl[i] = \
                [float(word) for word in line.split()[1:]]
        #---------------------------------------------------------------
        # pack into a dictionary 
        linecharacteristics[lchar-1] = \
            {'lchar'    : lchar,
             'nelseg'   : nelseg,
             'zfairlead': zfairlead,
             'zanchor'  : zanchor,
             'nsegments': nsegments,
             'slength'  : slength,
             'ibuoy'    : ibuoy,
             'slength'  : slength,
             'diameter' : diameter,
             'emod'     : emod,
             'emfact'   : emfact,
             'uwiw'     : uwiw,
             'watfac'   : watfac,
             'cdn'      : cdn,
             'cdl'      : cdl}
    
    #---------------------------------------------------------------
    linedata = \
        {'linechar'   : linechar,
         'xyfairlead' : xyfairlead,
         'xyanchor'   : xyanchor, 
         'heading'    : heading, 
         'tension'    : tension, 
         'horizontal' : horizontal,
         'winchlength': winchlength,
         'vesselheading': vesselheading,
         'vesselxoffset': vesselxoffset,
         'vesselyoffset': vesselyoffset}

    # GET BUOY DATA
    #-----------------------------------------
    locations = find_occurrences('ZBUOY (m)', reportlines)
    numberofbuoys = len(locations)
    buoyinfo      = [None]*numberofbuoys
    for i, loc in enumerate(locations): 
        zbuoy, fbuoy = reportlines[loc+1].split()
        buoyinfo[i] = (float(zbuoy), float(fbuoy))
    #---------------------------------------------------------------
    return (linedata, linecharacteristics, buoyinfo)





if __name__ == '__main__':
    sys.exit(main())



import numpy as np

def interpc(x, xp, fp):
  """ Alternative to numpy interp for complex numbers where the abs and phase are interpolated"""
  return np.interp(x, xp, np.abs(fp))*np.exp(1j*np.interp(x, xp, np.abs(fp)))